from typing import List

class RegexEngine:
    """
    RegexEngine is a class that compiles and manages a set of regular expression patterns.

    Attributes:
        raw_patterns (List[str]): A list of raw patterns as strings.
        compiled_patterns (List[re.Pattern]): A list of compiled regex patterns.
    """

    def __init__(self) -> None:
        """
        Initializes a new instance of RegexEngine.
        
        The `raw_patterns` attribute is initialized as an empty list to store raw regex patterns.
        The `compiled_patterns` attribute is initialized as an empty list to store compiled regex patterns.
        """
        ...

    def add_patterns(self, patterns: List[str], escape: bool) -> int:
        """
        Adds a list of patterns to the RegexEngine object. The patterns are compiled and stored in the object.

        Args:
            patterns (List[str]): A list of raw regex patterns to compile.
            escape (bool): A boolean flag to escape the patterns before compiling.

        Returns:
            int: The number of patterns that were successfully compiled.
        """
        ...
        
    @property
    def raw_patterns(self) -> List[str]:
        """
        Returns the raw patterns that were compiled.

        Returns:
            List[str]: A list of raw patterns that were compiled. The order of the patterns is the same as the order of the compiled patterns. The indexes of the raw patterns and the compiled patterns match.
        """
        ...

    def search(self, content: str) -> List[int]:
        """
        Runs the compiled patterns against the provided content.

        Args:
            content (str): The content to match against.

        Returns:
            List[int]: A list of indexes of the patterns that matched the content.
        """
        ...


import simple_fast_regex as sfr
import time

# Rust Implementation
engine = sfr.RegexEngine()

# Create one pattern that needs to escaping and one that doesn't
escape_pattern = "function addAllow(address holder, bool allowApprove) external onlyOwner { allow[holder] = allowApprove;"
no_escape_pattern = "if \\(owner == address\\(\\w+\\)\\) \\{ _allowances\\[owner\\]\\[spender\\] = amount; emit Approval\\(owner, spender, amount\\); \\} else \\{ _allowances\\[owner\\]\\[spender\\] = 0;"

# Add the patterns to the engine
engine.add_patterns(patterns=[escape_pattern], escape=True)
engine.add_patterns(patterns=[no_escape_pattern], escape=False)

# Run the search
TEXT0 = "No string should be found for this case"
TEXT1 = """function addAllow(address holder, bool allowApprove) external onlyOwner { allow[holder] = allowApprove;"""
TEXT2 = """if (owner == address(0xasida9sdu9asd)) { _allowances[owner][spender] = amount; emit Approval(owner, spender, amount); } else { _allowances[owner][spender] = 0;"""

matches = engine.search(TEXT0) # 0 matches
print(matches)

matches = engine.search(TEXT1) # 1 match
print(matches)

matches = engine.search(TEXT2) # 1 match
print(matches)

all_texts_together = TEXT0 + " \n " + TEXT1 + " \n " + TEXT2
matches = engine.search(all_texts_together) # 2 matches
print(matches)

# Print the raw_patterns list to make sure it's working fine
print(engine.raw_patterns)
